import { defineStore } from 'pinia'
import { v4 as uuidv4 } from 'uuid'
import { ref } from 'vue'

import { defaultFormations } from './formation-data'

export const useFormationStore = defineStore('formation', () => {
  const formations = ref(defaultFormations)

  function getFormation (id) {
    return formations.value.find(formation => formation.id === id)
  }

  function addFormation (formation) {
    if (!('id' in formation)) {
      formation.id = uuidv4()
      formations.value.push(formation)
    }
  }

  function deleteFormation (id) {
    formations.value = formations.value.filter(formation => formation.id !== id)
  }

  return {
    formations,
    getFormation,
    addFormation,
    deleteFormation
  }
}, { persist: true })
