import { vi, describe, it, beforeEach, afterEach, expect } from 'vitest'
import { mount } from '@vue/test-utils'
import { createTestingPinia } from '@pinia/testing'

import { useFormationStore } from '../../stores/formation.js'

import FormationsView from '../FormationsView.vue'
import { setActivePinia } from 'pinia'

describe('FormationsView', () => {
  let wrapper
  let formationStore

  beforeEach(async () => {
    console.log('== beforeEach ==')
    const thisPinia = createTestingPinia({
      stubActions: false,
      fakeApp: true,
      createSpy: vi.fn
    })
    setActivePinia(thisPinia)

    wrapper = mount(FormationsView, {
      global: {
        plugins: [thisPinia]
      }
    })
    if (formationStore) {
      console.log('formationStore.formations before useFormationStore')
      console.log(formationStore.formations)
      const testFormation = formationStore.formations.find(formation => formation.name === 'Test Formation')
      console.log(testFormation)
    }

    formationStore = useFormationStore()

    console.log('formationStore.formations after useFormationStore')
    console.log(formationStore.formations)
    const testFormation = formationStore.formations.find(formation => formation.name === 'Test Formation')
    console.log(testFormation)

    expect(wrapper.text()).not.toContain('Test Formation')
  })

  afterEach(async () => {
    wrapper.unmount()
  })

  describe('adding a formation', () => {
    let addModal

    beforeEach(async () => {
      console.log('== add object ==')
      await wrapper.get('#button-add').trigger('click')
      addModal = wrapper.find('#add-modal')
      await addModal.get('input[name=name]').setValue('Test Formation')
      await addModal.get('#button-save').trigger('click')
    })

    afterEach(async () => {
      console.log('== afterEach ==')
      expect(wrapper.text()).toContain('Test Formation')

      const addedFormation = formationStore.formations.find(formation => formation.name === 'Test Formation')
      const formationItem = wrapper.get(`#${addedFormation.id}`)

      console.log('before:')
      console.log(formationStore.getFormation(addedFormation.id))
      await formationItem.get('#button-delete').trigger('click')
      console.log('after:')
      console.log(formationStore.getFormation(addedFormation.id))

      const formationItemAfter = wrapper.find(`#${addedFormation.id}`)
      expect(formationItemAfter.exists()).toBe(false)
      expect(wrapper.text()).not.toContain('Test Formation')
      console.log('page does not contain Test Formation')

      const deletedFormation = formationStore.formations.find(formation => formation.name === 'Test Formation')
      expect(deletedFormation).toBeUndefined()
      console.log('store does not contain Test Formation')
    })

    it('the modal closes', () => {
      expect(wrapper.find('#add-modal').exists()).toBe(false)
    })

    it('the list item is added', () => {
      expect(wrapper.text()).toContain('Test Formation')
    })
  })
})
